
import numpy
import scipy.linalg
from apgl.util.Parameter import Parameter
from apgl.util.Util import Util 

"""
An implementation of the Primal PCA algorithm. 
"""

class PrimalPCA(object):
    def __init__(self, k):
        """
        Create the PCA object
        """
        pass 

    def learnModel(self, X):
        """
        Learn the PCA directions.

        :param testX: The training examples organised into rows.
        :type testX: :class:`numpy.ndarray`
        """
        Parameter.checkClass(X, numpy.ndarray)

        C = numpy.dot(X.T, X)
        (lmbdas, U) = scipy.linalg.eig(C)

        self.lmbdas = lmbdas
        self.U = U

        return self.U, self.lmbdas

    def project(self, testX, k=None):
        """
        Project the examples into the PCA space using k directions, or all if k=None.

        :param testX: The examples to project given as rows of a matrix.
        :type testX: :class:`numpy.ndarray`

        :param k: The number of projection directions.
        :type k: :class:`int`

        :returns: The projected examples.
        """
        Parameter.checkClass(testX, numpy.ndarray)

        if k==None:
            return numpy.dot(testX, self.U)
        else:
            Parameter.checkInt(k, 1, float('inf'))
            return numpy.dot(testX, self.U[:, 0:k])

