# To change this template, choose Tools | Templates
# and open the template in the editor.

import unittest
import numpy
import scipy.linalg
from apgl.features.PrimalPCA import PrimalPCA 


class  PrimalPCATest(unittest.TestCase):
    def setUp(self):
        pass

    def testLearnModel(self):
        numExamples = 10
        numFeatures = 5

        X = numpy.random.randn(numExamples, numFeatures)

        k = 4
        tol = 10**-6 
        pca = PrimalPCA(k)
        U, lmbdas = pca.learnModel(X)

        #Compute eignvalues manually
        C = numpy.dot(X.T, X)
        lmbdas2, U2 = scipy.linalg.eig(C)

        self.assertTrue( (lmbdas >= 0).all())
        self.assertEquals(lmbdas.shape[0], numFeatures )
        self.assertEquals(U.shape[1], numFeatures )

        self.assertTrue( numpy.linalg.norm(lmbdas2- lmbdas) <= tol )
        self.assertTrue( numpy.linalg.norm(U2- U) <= tol )

        newX = pca.project(X)
        newX2 = numpy.dot(X, U2)

        self.assertTrue( numpy.linalg.norm(newX2 - newX) <= tol )

        #Test orthogonality of U
        self.assertTrue( numpy.linalg.norm(numpy.dot(U.T, U) - numpy.eye(numFeatures)) <= tol )

        #Test projecting just k directions
        k = 2
        newX = pca.project(X, k)
        self.assertTrue( numpy.linalg.norm(newX2[:, 0:k] - newX) <= tol)


if __name__ == '__main__':
    unittest.main()

