
from apgl.graph.SparseGraph import SparseGraph
from apgl.graph.GeneralVertexList import GeneralVertexList
from apgl.generator.ConfigModelGenerator import ConfigModelGenerator
import unittest
import numpy 

class ConfigModelGeneratorTest(unittest.TestCase):
    def setUp(self):
        pass

    def testInit(self):
        degSequence = numpy.array([2, 1, 2])
        generator = ConfigModelGenerator(degSequence)

        degSequence = numpy.array([2, 1, 3])
        generator = ConfigModelGenerator(degSequence)

        degSequence = numpy.array([2, 1, 2.2])
        generator = self.assertRaises(ValueError, ConfigModelGenerator, degSequence)

        degSequence = numpy.array([2, 1, -1])
        generator = self.assertRaises(ValueError, ConfigModelGenerator, degSequence)

        degSequence = numpy.array([2, 1, 4])
        generator = self.assertRaises(ValueError, ConfigModelGenerator, degSequence)

    def testGenerate(self):
        degSequence = numpy.array([2, 1, 3, 0, 0, 0, 0, 0, 0, 1])
        generator = ConfigModelGenerator(degSequence)

        numVertices = 10
        graph = SparseGraph(GeneralVertexList(numVertices))
        graph = generator.generate(graph)

        tol = 2
        self.assertTrue(numpy.linalg.norm(degSequence - graph.degreeSequence()) < tol)

        degSequence = numpy.array([2, 1, 3, 0, 2, 1, 4, 0, 0, 1])
        generator.setDegSequence(degSequence)
        graph.removeAllEdges()
        graph = generator.generate(graph)

        self.assertTrue(numpy.linalg.norm(degSequence - graph.degreeSequence()) < tol)

        #Test using a non-empty graph
        degSequence = numpy.array([0, 0, 0, 2, 0, 0, 0, 1, 1, 0])
        generator.setDegSequence(degSequence)
        oldDegSequence = graph.degreeSequence()

        self.assertRaises(ValueError, generator.generate, graph, True)
        graph = generator.generate(graph, False)

        diffSequence = graph.degreeSequence() - oldDegSequence

        self.assertTrue(numpy.linalg.norm(degSequence - diffSequence) < tol)

if __name__ == '__main__':
    unittest.main()