'''
Created on 1 Jul 2009

@author: charanpal
'''

from apgl.util.PathDefaults import PathDefaults 
from apgl.graph.VertexList import VertexList
import unittest
import numpy 

class VertexListTest(unittest.TestCase):
    def setUp(self):
        self.numVertices = 10 
        self.numFeatures = 3
     
        self.vList = VertexList(self.numVertices, self.numFeatures)
        
    def tearDown(self):
        pass

    def testInit(self):
        pass        
        #self.assertRaises(ValueError, MarkovProcess, transitionMatrix1) 

    def testConstructor(self):
        self.assertEquals(self.vList.getNumFeatures(), self.numFeatures)
        self.assertEquals(self.vList.getNumVertices(), self.numVertices)
        
    def testSetVertex(self):
        ind = 3;
        value = numpy.array([0.1, 0.5, 0.1])
        self.vList.setVertex(ind, value)
        
        self.assertEquals((self.vList.getVertex(ind) == value).all(), 1)
        
    def testClearVertex(self):
        ind = 3;
        value = numpy.array([0.1, 0.5, 0.1])
        self.vList.setVertex(ind, value)
        
        self.assertEquals((self.vList.getVertex(ind) == value).all(), 1)
        
        z = numpy.zeros((1, 3))
        self.vList.clearVertex(3)
        self.assertEquals((self.vList.getVertex(ind) == z).all(), 1)
        
    def testGetFeatureDistribution(self):
        numVertices = 10 
        numFeatures = 3
     
        vList = VertexList(numVertices, numFeatures)
        value = numpy.array([0.1, 0.5, 0.1])
        vList.setVertex(0, value)
        vList.setVertex(1, value)
        vList.setVertex(2, value)
        
        (freqs, items) = vList.getFeatureDistribution(0)
        self.assertTrue((freqs == numpy.array([7, 3])).all())
        (freqs, items) = vList.getFeatureDistribution(1)
        self.assertTrue((freqs == numpy.array([7, 3])).all())
        (freqs, items) = vList.getFeatureDistribution(2)
        self.assertTrue((freqs == numpy.array([7, 3])).all())
        
        #Test error cases 
        self.assertRaises(ValueError, self.vList.getFeatureDistribution, -1)
        self.assertRaises(ValueError, self.vList.getFeatureDistribution, 3)
        
        
        vList.setVertex(3, numpy.array([0.5, 0.5, 0.1]))
        (freqs, items) = vList.getFeatureDistribution(0)
        self.assertTrue((freqs == numpy.array([6, 3, 1])).all())
        
        vList.setVertex(4, numpy.array([0.6, 0.5, 0.1]))
        (freqs, items) = vList.getFeatureDistribution(0)
        self.assertTrue((freqs == numpy.array([5, 3, 1, 1])).all())

    def testGetVertices(self):
        vList = VertexList(self.numVertices, self.numFeatures)
        vList.setVertex(0, numpy.array([1, 2, 3]))
        vList.setVertex(1, numpy.array([4, 5, 6]))
        vList.setVertex(2, numpy.array([7, 8, 9]))
        vList.setVertex(3, numpy.array([0, 11, 12]))

        self.assertTrue((vList.getVertices([0, 3]) == numpy.array([[1, 2, 3], [0, 11, 12]])).all())
        self.assertTrue((vList.getVertices([3, 0]) == numpy.array([[0, 11, 12], [1, 2, 3]])).all())
        self.assertTrue((vList.getVertices([0, 2]) == numpy.array([[1, 2, 3], [7, 8, 9]])).all())
        self.assertTrue((vList.getVertices([1]) == numpy.array([[4,5,6]])).all())

        #Test returning all vertices
        self.assertTrue((vList.getVertices() == vList.V).all())


    def testGetCopy(self):
        vList = VertexList(self.numVertices, self.numFeatures)
        vList.setVertex(0, numpy.array([1, 2, 3]))
        vList.setVertex(1, numpy.array([4, 5, 6]))
        vList.setVertex(2, numpy.array([7, 8, 9]))

        vList2 = vList.copy()
        self.assertTrue((vList.getVertices(list(range(0, self.numVertices))) == vList2.getVertices(list(range(0, self.numVertices)))).all())

        vList.setVertex(0, numpy.array([10, 20, 30]))
        self.assertTrue((vList2.getVertex(0) == numpy.array([1, 2, 3])).all())

    def testSubList(self):
        vList = VertexList(self.numVertices, self.numFeatures)
        vList.setVertex(0, numpy.array([1, 2, 3]))
        vList.setVertex(1, numpy.array([4, 5, 6]))
        vList.setVertex(2, numpy.array([7, 8, 9]))

        vList2 = vList.subList([0, 2])

        self.assertEquals(vList2.getNumVertices(), 2)
        self.assertTrue((vList.getVertices([0, 2]) == vList2.getVertices([0, 1])).all())

    def testSaveLoad(self):
        try:
            vList = VertexList(self.numVertices, self.numFeatures)
            vList.setVertex(0, numpy.array([1, 2, 3]))
            vList.setVertex(1, numpy.array([4, 5, 6]))
            vList.setVertex(2, numpy.array([7, 8, 9]))

            tempDir = PathDefaults.getTempDir()
            fileName = tempDir + "vList"

            vList.save(fileName)
            vList2 = VertexList.load(fileName)

            self.assertTrue((vList.getVertices() == vList2.getVertices()).all())
        except IOError as e:
            print(e)
            pass 


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(VertexListTest)
    unittest.TextTestRunner(verbosity=2).run(suite)
    