'''
Created on 24 Jul 2009

@author: charanpal
'''
import unittest
import numpy 
import logging
import apgl
from apgl.predictors.MlpySVM import MlpySVM
from apgl.data.ExamplesGenerator import ExamplesGenerator

@apgl.skipIf(not apgl.checkImport('mlpy'), 'Module mlpy is required')
class MlpySVMTest(unittest.TestCase):
    def setUp(self):
        try:
            import mlpy
            numExamples = 100
            numFeatures = 10
            eg = ExamplesGenerator()

            self.X, self.y = eg.generateBinaryExamples(numExamples, numFeatures)
            self.svm = MlpySVM()
        except ImportError:
            logging.debug("Mlpy is required to run this test script.")

    def tearDown(self):
        pass

    def testLearnModel(self):
        try:
            import mlpy

            self.svm.learnModel(self.X, self.y)
            predY = self.svm.classify(self.X)

            e = mlpy.err(predY, self.y)
            logging.debug("Error rate : "  + str(e))

            #Test an empty example list - should raise a value error in SVM code
            numExamples = 0
            numFeatures = 1
            eg = ExamplesGenerator()

            X, y = eg.generateBinaryExamples(numExamples, numFeatures)
            self.assertRaises(ValueError, self.svm.learnModel, X, y)
        except ImportError:
            pass 
        
    def testClassify(self):
        try:
            import mlpy
            self.svm.learnModel(self.X, self.y)
            predY = self.svm.classify(self.X)

            e = mlpy.err(predY, self.y)

            #Now, permute examples
            inds = numpy.random.permutation(self.X.shape[0])
            self.X = self.X[inds, :]
            self.y = self.y[inds]
            predY = self.svm.classify(self.X, self.y)

            e2 = mlpy.err(predY, self.y)

            self.assertEquals(e, e2)
        except ImportError:
            pass
    
    #Quite a simple test   
    #Followed up by printing and examining intermediate variables in the method   
    def testCvModelSelection(self):
        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [0.1, 0.5]
        folds = 5 
        
        C, kernelParam, errorCost, error = self.svm.cvModelSelection(self.X, self.y, Cs, kernelParams, kernel, folds, errorCosts)
        
        self.assertTrue(C in Cs)
        self.assertTrue(kernelParam in kernelParams)
        self.assertTrue(error >= 0 and error <= 1)
        
        #Test gaussian kernel 
        kernel= "gaussian"
        
        C, kernelParam, errorCost, error = self.svm.cvModelSelection(self.X, self.y, Cs, kernelParams, kernel, folds, errorCosts)
        
        self.assertTrue(C in Cs)
        self.assertTrue(kernelParam in kernelParams)
        self.assertTrue(error >= 0 and error <= 1)
    
    def testEvaluateCv(self):
        folds = 10
        (means, vars) = self.svm.evaluateCv(self.X, self.y, folds)

        self.assertEquals(means.shape[0], 6)
        self.assertEquals(vars.shape[0], 6)

        self.assertTrue((means <= 1).all())
        self.assertTrue((means>= 0).all())
        self.assertTrue((vars <= 1).all())
        self.assertTrue((vars>= 0).all())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()