"""
Use an eigen decomposition update based on the SVD update.
"""
import numpy
from apgl.util.Util import Util 

numpy.set_printoptions(suppress=True, precision=3, linewidth=300)
numpy.random.seed(21)
numpy.seterr(all='raise')

def eigenUpdate(Q, s, AB, BB):
    """
    Find the eigen update of a matrix [A, B]'[A B] where  A'A = V diag(s) V.T
    and AB = A'B, BB = B'B
    """
    m = Q.shape[0]
    k = Q.shape[1]
    p = BB.shape[0]

    L = numpy.diag(s**-1)
    AA = Q.dot(L).dot(Q.conj().T)

    QAB = Q.conj().T.dot(AB)
    #print(ABk)

    BBhat = BB - QAB.conj().T.dot(L).dot(QAB)
    BBhat = numpy.array(BBhat, numpy.complex)

    lHat, Qhat = numpy.linalg.eig(BBhat)

    E = numpy.r_[numpy.diag(s), QAB.conj().T]
    F = numpy.r_[QAB.conj(), QAB.conj().T.dot(L).dot(QAB) + BBhat]
    E = numpy.c_[E, F]

    G = numpy.r_[Q, numpy.zeros((p, k))]
    H = numpy.r_[numpy.zeros((m, p)), numpy.eye(p)]

    G = numpy.c_[G, H]

    gammaSq, V = numpy.linalg.eig(E)
    inds = numpy.flipud(numpy.argsort(numpy.abs(gammaSq)))
    V = V[:, inds[0:k]]
    gammaSq = gammaSq[inds[0:k]]

    Stilde = numpy.diag(gammaSq)
    Vtilde = numpy.dot(G, V)

    return Stilde, Vtilde

m = 10
n = 10
r = 5
k = 6
A = numpy.random.rand(m, n)
B = numpy.random.rand(m, r)

AcB = numpy.c_[A, B]
ABBA = (AcB.T.dot(AcB))
D = numpy.diag(ABBA)
ABBA = ABBA - numpy.diag(D)

AA = ABBA[0:n, 0:n]
AB = ABBA[0:n, n:]
BB = ABBA[n:, n:]

#First compute eigen update estimate 
s, V = numpy.linalg.eig(AA)
inds = numpy.flipud(numpy.argsort(numpy.abs(s)))
V = V[:, inds[0:k]]
s = s[inds[0:k]]
Stilde, Vtilde = eigenUpdate(V, s, AB, BB)
ABBAEst = numpy.dot(numpy.dot(Vtilde, Stilde), Vtilde.T)

#Compute estimate using eigendecomposition of full matrix
s2, V2 = numpy.linalg.eig(ABBA)
inds2 = numpy.flipud(numpy.argsort(numpy.abs(s2)))
V2 = V2[:, inds2[0:k]]
s2 = s2[inds2[0:k]]
ABBAEst2 = V2.dot(numpy.diag(s2)).dot(V2.T)

print(numpy.linalg.norm(ABBAEst - ABBA))
print(numpy.linalg.norm(ABBAEst2 - ABBA))
print(numpy.linalg.norm(ABBA))

#It seems to work even with non positive definite matrices!