
"""
A new approach to spectral clustering based on iterative eigen decomposition.
"""
import numpy
import scipy.cluster.vq
from apgl.sandbox.EigenUpdate import eigenUpdate
from apgl.graph import * 

numpy.set_printoptions(suppress=True, precision=3, linewidth=200)

#First, compute two adjacency matrices and then find update 
numVertices = 10

graph = DenseGraph(GeneralVertexList(numVertices))
graph.addEdge(0, 1)
graph.addEdge(0, 2)
graph.addEdge(1, 3)
graph.addEdge(2, 3)
graph.addEdge(1, 2)
graph.addEdge(3, 4)
graph.addEdge(4, 5)
graph.addEdge(4, 7)
graph.addEdge(5, 6)
graph.addEdge(6, 7)
graph.addEdge(5, 7)
graph.addEdge(7, 9)
graph.addEdge(2, 8)

W = graph.getWeightMatrix()

#Split the graph into 2 
AA = W[0:8, 0:8]
AB = W[0:8, 8:]
BB = W[8:, 8:]

k = 2

s, U = numpy.linalg.eig(AA)
inds = numpy.flipud(numpy.argsort(numpy.abs(s)))
U = U[:, inds[0:k]]
s = s[inds[0:k]]
s = numpy.array(s, numpy.complex)

#Now diagonalise AA
d = numpy.sum(AA, 0)
D = numpy.diag(d)
Uhat = U.dot(numpy.diag(s**-0.5))
Z = Uhat.T.dot(D).dot(Uhat)

s2, Y = numpy.linalg.eig(Z)
V = Uhat.dot(Y)
V = V.dot(numpy.diag(numpy.diag(V.T.dot(V))**-0.5))

#Test if V is correct
#Now normalise rows of V and use for k means
V = numpy.diag(numpy.diag(V.dot(V.T))**-0.5).dot(V)
V = numpy.real(V)

print(V)
#Whiten doesn't seem to work 
#V = scipy.cluster.vq.whiten(V)
V = V.dot(numpy.diag(numpy.diag(V.T.dot(V))**-0.5))

print(V)

iterations = 100 
centroid, label = scipy.cluster.vq.kmeans2(V, k, iterations)
print(label)


#Now update clusters 
sPrime, Uprime = eigenUpdate(U, s, AB, BB)
sPrime = numpy.diag(sPrime)
inds = numpy.flipud(numpy.argsort(numpy.abs(sPrime)))
Uprime = Uprime[:, inds[0:k]]
sPrime = sPrime[inds[0:k]]
sPrime = numpy.array(sPrime, numpy.complex)

#Now diagonalise [AA AB; BA BB]
d = numpy.sum(W, 0)
D = numpy.diag(d)
Uhat = Uprime.dot(numpy.diag(sPrime**-0.5))
Z = Uhat.T.dot(D).dot(Uhat)

print(D.shape)

s2, Y = numpy.linalg.eig(Z)
V = Uhat.dot(Y)
V = V.dot(numpy.diag(numpy.diag(V.T.dot(V))**-0.5))

V = numpy.diag(numpy.diag(V.dot(V.T))**-0.5).dot(V)
V = numpy.real(V)
V = V.dot(numpy.diag(numpy.diag(V.T.dot(V))**-0.5))

print(V)

centroid, label = scipy.cluster.vq.kmeans2(V, k, iterations)
print(label)

#Works - hurray!
#Now write general function

def incSpectralClustering():
    pass 
    #TODO 