
import numpy
from apgl.util.Parameter import Parameter
from apgl.util.Util import Util
from apgl.graph.VertexList import VertexList
from apgl.graph.SparseGraph import SparseGraph

class KroneckerEpidemicGenerator(object):
    '''
    A class which generates graphs according to the Kronecker Epidemic method.
    '''
    def __init__(self, initialGraph, k):
        """
        Initialise with a starting graph, and number of iterations k and decay
        factor n. 
        """
        Parameter.checkInt(k, 1, float('inf'))
        self.initialGraph = initialGraph
        self.k = k

    def setK(self, k):
        Parameter.checkInt(k, 1, float('inf'))
        self.k = k

    def generateGraph(self):
        """
        Generate an Epidemic Kronecker graph
        """
        W = self.initialGraph.getWeightMatrix()

        if (numpy.diag(W)==numpy.zeros(W.shape[0])).any():
            raise ValueError("Initial graph must have all self-edges")

        a = numpy.ones(W.shape[0])
        ai = numpy.ones(W.shape[0])
        
        Wi = W
        q = 1

        for i in range(1, self.k):
            Wi = numpy.kron(Wi, W)
            ji = numpy.ones(Wi.shape[0])
            ai = numpy.r_[ai, numpy.zeros(ai.shape[0]*(a.shape[0]-1))] + ji
            #print(ai)

        A = numpy.abs(numpy.outer(ai, ji) - numpy.outer(ji, ai)) <= q
        #print(A)
        Wi = Wi * A

        vList = VertexList(Wi.shape[0], 0)
        graph = SparseGraph(vList, self.initialGraph.isUndirected())
        graph.setWeightMatrix(Wi)

        return graph