

"""
Some functions to make profiling code a bit simpler.
"""

import logging
from apgl.util.PathDefaults import PathDefaults

class ProfileUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def profile(command, globalVars, localVars, numStats=30):
        """
        Just profile the given command with the global and local variables
        and print out the cumulative and function times. 
        """
        try:
            import pstats
            import cProfile
        except ImportError:
            raise ImportError("profile() requires pstats and cProfile")

        outputDirectory = PathDefaults.getOutputDir()
        directory = outputDirectory + "test/"
        profileFileName = directory + "profile.cprof"

        logging.info("Starting to profile ...")
        cProfile.runctx(command, globalVars, localVars, profileFileName)
        logging.info("Done")
        stats = pstats.Stats(profileFileName)
        stats.strip_dirs().sort_stats("cumulative").print_stats(numStats)
        stats.strip_dirs().sort_stats("time").print_stats(numStats)
