


import numpy 
import unittest
from apgl.util import * 


class  EvaluatorTestCase(unittest.TestCase):
    #def setUp(self):
    #    self.foo = Evaluator()
    #


    def testRootMeanSqError(self):
        y = numpy.array([1,2,3])
        predY = numpy.array([1,2,3])

        self.assertEquals(Evaluator.rootMeanSqError(y, predY), 0.0)

        y = numpy.array([1,2,3])
        predY = numpy.array([1,2,2])

        self.assertEquals(Evaluator.rootMeanSqError(y, predY), float(1)/numpy.sqrt(3))

        predY = numpy.array([1,2])
        self.assertRaises(ValueError, Evaluator.rootMeanSqError, y, predY)

    def testWeightedRootMeanSqError(self):

        y = numpy.array([0.1, 0.2, 0.3])
        predY = numpy.array([0.1, 0.2, 0.3])

        self.assertEquals(Evaluator.weightedRootMeanSqError(y, predY), 0.0)

        #Errors on larger ys are weighted more 
        predY = numpy.array([0.0, 0.2, 0.3])
        predY2 = numpy.array([0.1, 0.2, 0.4])

        self.assertTrue(Evaluator.weightedRootMeanSqError(y, predY) < Evaluator.weightedRootMeanSqError(y, predY2))

    def testBinaryError(self):
        testY = numpy.array([1, 1, -1, 1])
        predY = numpy.array([-1, 1, -1, 1])
        predY2 = numpy.array([-1, -1, -1, 1])
        predY3 = numpy.array([-1, -1, 1, -1])

        self.assertTrue(Evaluator.binaryError(testY, predY) == 0.25)
        self.assertTrue(Evaluator.binaryError(testY, testY) == 0.0)
        self.assertTrue(Evaluator.binaryError(predY, predY) == 0.0)

        self.assertTrue(Evaluator.binaryError(testY, predY2) == 0.5)
        self.assertTrue(Evaluator.binaryError(testY, predY3) == 1.0)

    def testBinaryErrorP(self):
        testY = numpy.array([1, 1, -1, 1])
        predY = numpy.array([-1, 1, -1, 1])
        predY2 = numpy.array([-1, -1, -1, 1])
        predY3 = numpy.array([-1, -1, 1, -1])

        self.assertTrue(Evaluator.binaryErrorP(testY, predY) == 1.0/3.0)
        self.assertTrue(Evaluator.binaryErrorP(testY, testY) == 0.0)
        self.assertTrue(Evaluator.binaryErrorP(predY, predY) == 0.0)

        self.assertTrue(Evaluator.binaryErrorP(testY, predY2) == 2.0/3.0)
        self.assertTrue(Evaluator.binaryErrorP(testY, predY3) == 1.0)

        testY = numpy.array([-1, -1, -1, -1])
        predY = numpy.array([-1, 1, -1, 1])

        self.assertTrue(Evaluator.binaryErrorP(testY, predY) == 0.0)

    def testBinaryErrorN(self):
        testY = numpy.array([1, 1, -1, 1])
        predY = numpy.array([-1, 1, -1, 1])
        predY2 = numpy.array([-1, -1, -1, 1])
        predY3 = numpy.array([-1, -1, 1, -1])

        self.assertTrue(Evaluator.binaryErrorN(testY, predY) == 0.0)
        self.assertTrue(Evaluator.binaryErrorN(testY, testY) == 0.0)
        self.assertTrue(Evaluator.binaryErrorN(predY, predY) == 0.0)

        self.assertTrue(Evaluator.binaryErrorN(testY, predY2) == 0.0)
        self.assertTrue(Evaluator.binaryErrorN(testY, predY3) == 1.0)

        testY = numpy.array([-1, -1, -1, -1])
        predY = numpy.array([-1, 1, -1, 1])

        self.assertTrue(Evaluator.binaryErrorN(testY, predY) == 0.5)

        testY = numpy.array([1, 1, 1, 1])
        predY = numpy.array([-1, 1, -1, 1])

        self.assertTrue(Evaluator.binaryErrorN(testY, predY) == 0.0)

if __name__ == '__main__':
    unittest.main()

