"""
Take all the edges as a flat list of examples and find the best parameters for
a classifier, and then make a prediction. 
"""


import logging
import sys
import numpy
from apgl.util import *
from apgl.graph import *
from apgl.predictors import *
from apgl.kernel import *
from apgl.viroscopy.HIVGraphReader import HIVGraphReader
from apgl.data.Standardiser import Standardiser

logging.basicConfig(stream=sys.stdout, level=logging.INFO)

hivReader = HIVGraphReader()
graph = hivReader.readHIVGraph()

#The set of edges indexed by zeros is the contact graph
#The ones indexed by 1 is the infection graph
edgeTypeIndex1 = 0
edgeTypeIndex2 = 1
sGraphContact = graph.getSparseGraph(edgeTypeIndex1)
sGraphInfect = graph.getSparseGraph(edgeTypeIndex2)
sGraphContact = sGraphContact.union(sGraphInfect)

graph = sGraphContact

#Take a subgraph 
#perm = numpy.random.permutation(graph.getNumVertices())[0:1000]
#graph = graph.subgraph(perm)

"""
Construct the full set of examples 
"""

X, y = GraphUtils.vertexLabelExamples(graph)
perm = numpy.random.permutation(X.shape[0])
X = X[perm, :]
y = y[perm]

processor = Standardiser()
X = processor.standardiseArray(X)

numExamples = X.shape[0]
numTrainExamples = numpy.floor(numExamples/2)
numTestExamples = numExamples - numTrainExamples

trainX = X[0:numTrainExamples, :]
trainY = y[0:numTrainExamples]
testX = X[numTrainExamples:, ]
testY = y[numTrainExamples:]

svm = LibSVM()

#First try the RBF SVM
Cs = (2**numpy.arange(-4, 5, dtype=numpy.float)).tolist()
kernel = "gaussian"
kernelParams = (2**numpy.arange(-4, 5, dtype=numpy.float)).tolist()
folds = 5
errorCosts = [1.0]

best = svm.cvModelSelection(trainX, trainY, Cs, kernelParams, kernel, folds, errorCosts)
print(best)

fileName = PathDefaults.getOutputDir() + "ViroGaussianSVMParams"
svm.setKernel("gaussian", best["kp"])
svm.setC(best["C"])
svm.saveParams(fileName)

(mean, var) = svm.evaluateCv(testX, testY, folds)
print("\n")
print(("Error using " + str(testX.shape[0]) + " examples: " + str(mean) + "(" + str(var) + ")"))

#Test with the linear SVM
Cs = (2**numpy.arange(-4, 5, dtype=numpy.float)).tolist()
kernel = "linear"
kernelParams = [1]
folds = 5
errorCosts = [1.0]

best = svm.cvModelSelection(trainX, trainY, Cs, kernelParams, kernel, folds, errorCosts)

print(best)
svm.setKernel("linear", 1)
svm.setC(best["C"])
fileName = PathDefaults.getOutputDir() + "ViroLinearSVMParams"
svm.saveParams(fileName)

(mean, var) = svm.evaluateCv(testX, testY, folds)

print("\n")
print(("Error using " + str(testX.shape[0]) + " examples: " + str(mean) + "(" + str(var) + ")"))

#Now print out the SVM weights by learning on all data 
svm.learnModel(X, y)

weights, rho = svm.getWeights()
print("SVM weights in rank order: ")
print((numpy.flipud(numpy.argsort(numpy.abs(weights)))))
print((weights[numpy.flipud(numpy.argsort(numpy.abs(weights)))]))