"""
Here we generate a number of graphs according to the configuration model and fit
the degree sequence to the HIV graph. 
"""
import logging
import sys
import numpy
from apgl.graph import *
from apgl.viroscopy.HIVGraphReader import HIVGraphReader
from apgl.util.DateUtils import DateUtils
from apgl.util.PathDefaults import PathDefaults 
from apgl.generator.ConfigModelGenerator import ConfigModelGenerator

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
numpy.set_printoptions(suppress=True, linewidth=60)
numpy.random.seed(21)

hivReader = HIVGraphReader()
graph = hivReader.readHIVGraph()
fInds = hivReader.getIndicatorFeatureIndices()

#The set of edges indexed by zeros is the contact graph
#The ones indexed by 1 is the infection graph
edgeTypeIndex1 = 0
edgeTypeIndex2 = 1
sGraphContact = graph.getSparseGraph(edgeTypeIndex1)
sGraphInfect = graph.getSparseGraph(edgeTypeIndex2)
sGraphContact = sGraphContact.union(sGraphInfect)
sGraph = sGraphContact

startYear = 1900
daysInYear = 365
daysInMonth = 30 
monthStep = 3

detectionIndex = fInds["detectDate"]
vertexArray = sGraph.getVertexList().getVertices()
detections = vertexArray[:, detectionIndex]

#Make sure we include all detections
dayList = range(int(numpy.min(detections)), int(numpy.max(detections)), daysInMonth*monthStep)
dayList.append(numpy.max(detections))

#The config graph has the same number of edges and vertices at each time point
numGraphs = 5
numVertices = sGraph.getNumVertices()

resultsDir = PathDefaults.getOutputDir() + "viroscopy/"
graphFileNameBase = resultsDir + "ConfigGraph"

for j in range(numGraphs):
    configGraph = SparseGraph(GeneralVertexList(numVertices))
    degSequence = numpy.zeros(numVertices, numpy.int)
    lastDegSequence = numpy.zeros(numVertices, numpy.int)
    generator = ConfigModelGenerator(lastDegSequence)

    for i in dayList:
        logging.info("Date: " + str(DateUtils.getDateStrFromDay(i, startYear)))
        subgraphIndices = numpy.nonzero(detections <= i)[0]
        subgraphIndices = numpy.unique(subgraphIndices)
        subgraph = sGraph.subgraph(subgraphIndices)

        subDegSequence = subgraph.degreeSequence()
        degSequence[subgraphIndices] = subDegSequence
        diffSequence = degSequence - lastDegSequence
        generator.setDegSequence(diffSequence)
        configGraph = generator.generate(configGraph, False)

        lastDegSequence = configGraph.degreeSequence()

    configGraph.save(graphFileNameBase + str(j))