"""
A script to estimate the HIV epidemic model parameters using ABC.
"""
from apgl.graph import *
from apgl.util import *
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVABCParameters import HIVABCParameters
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVRates import HIVRates
from apgl.predictors.ABCSMC import ABCSMC

import logging
import sys
import numpy
import multiprocessing

FORMAT = "%(levelname)s:root:%(process)d:%(message)s"
logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=FORMAT)
numpy.set_printoptions(suppress=True, precision=3)
numpy.seterr(invalid='raise')

#We will load up the statistics from the real graph
resultsDir = PathDefaults.getOutputDir() + "viroscopy/"
#resultsFileName = resultsDir + "ContactGrowthScalarStats.pkl"
resultsFileName = resultsDir + "SimContactGrowthScalarStats.pkl"
statsArray = Util.loadPickle(resultsFileName)

recordStep = 90
timesFileName = resultsDir + "epidemicTimes.pkl"
dayList = Util.loadPickle(timesFileName)
numTimes = len(dayList)
T = float(dayList[-1]+1)

graphStats = GraphStatistics()
realValues = statsArray[list(range(len(dayList))), graphStats.numVerticesIndex]
logging.info("dayList=" + str(dayList))
logging.info("realValues=" + str(realValues))
diffValues = numpy.diff(realValues)
logging.info("diffValues=" + str(diffValues))
epsilonArray = numpy.linspace(numpy.linalg.norm(diffValues)/2, numpy.linalg.norm(diffValues)/5, 3)
logging.info("epsilonArray=" + str(epsilonArray))

def createModel(t):
    """
    The parameter t is the particle index. 
    """
    M = 1000
    printStep = 50
    undirected = True

    graph = HIVGraph(M, undirected)
    logging.info("Created graph: " + str(graph))
    rates = HIVRates(graph)
    abcParams = HIVABCParameters(graph, rates)
    
    model = HIVEpidemicModel(graph, rates)
    model.setT(T)
    model.setRecordStep(recordStep)
    model.setPrintStep(printStep)
    model.setBreakFunction(abcParams.getBreakFunc(realValues, epsilonArray[t]))

    return model, abcParams

numProcesses = multiprocessing.cpu_count()
#numProcesses = 1
posteriorSampleSize = 50
thetaLen = 11 

#Create shared variables 
thetaQueue = multiprocessing.Queue()
distQueue = multiprocessing.Queue()
summaryQueue = multiprocessing.Queue()
args = (thetaQueue, distQueue, summaryQueue)
abcList = []

thetaFileName = resultsDir + "thetaReal.pkl"
theta = Util.loadPickle(thetaFileName)
logging.info("Real theta values: " + str(theta))

for i in range(numProcesses):
    abcList.append(ABCSMC(args, epsilonArray, realValues, createModel))
    abcList[i].setPosteriorSampleSize(posteriorSampleSize)
    abcList[i].start()

logging.info("All processes started")

for i in range(numProcesses):
    abcList[i].join()

logging.info("Queue size = " + str(thetaQueue.qsize()))
thetasArray = numpy.zeros((thetaQueue.qsize(), thetaLen))

for i in range(thetaQueue.qsize()):
    thetasArray[i, :] = numpy.array(thetaQueue.get())

meanTheta = numpy.mean(thetasArray, 0)
stdTheta = numpy.std(thetasArray, 0)
logging.info(thetasArray)
logging.info("meanTheta=" + str(meanTheta))
logging.info("stdTheta=" + str(stdTheta))

thetaFileName =  resultsDir + "thetaDistSimulated.pkl"
Util.savePickle(thetasArray, thetaFileName)

    
#TODO: Check the new contacts being made is realisitic range