
from apgl.graph import *
from apgl.util import *
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVRates import HIVRates
from apgl.viroscopy.model.HIVABCParameters import HIVABCParameters

import logging
import sys
import numpy

"""
This is the epidemic model for the HIV spread in cuba. We repeat the simulation a number
of times and average the results. 
"""

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
numpy.seterr(all='raise')
numpy.random.seed(24)

T = 1000.0
recordStep = 90
printStep = 10
M = 1000
numRepetitions = 10
undirected = True

outputDir = PathDefaults.getOutputDir() + "viroscopy/"

#Default Params based on real data
#theta = [50, 1.0, 2.0, 1.0/800, 0.01, 0.1, 0.2, 34.0/10000, 38.0/10000, 30.0/10000, 170.0/10000]
#Default Params 2 for demonstrating model
theta = [50, 1.0, 2.0, 1.0/800, 0.01, 0.1, 0.2, 34.0/1000, 38.0/1000, 30.0/1000, 170.0/1000]
#theta = [50, 1.0, 2.0, 0.0, 0.00, 0.1, 0.2, 34.0/1000, 38.0/1000, 30.0/1000, 170.0/1000]
#Params 3 for showing properties of detection graph
#theta = [100, 1.0, 2.0, 1.0/800, 0.01, 0.1, 0.2, 34.0/1000, 38.0/1000, 30.0/1000, 500.0/1000]
thetaFileName = outputDir + "thetaReal.pkl"
Util.savePickle(theta, thetaFileName)


for j in range(numRepetitions):
    graph = HIVGraph(M, undirected)
    logging.info("Created graph: " + str(graph))
    rates = HIVRates(graph)
    model = HIVEpidemicModel(graph, rates)
    model.setT(T)
    model.setRecordStep(recordStep)
    model.setPrintStep(printStep)

    params = HIVABCParameters(graph, rates)
    paramFuncs = params.getParamFuncs()

    for i in range(len(theta)):
        print(paramFuncs[i])
        paramFuncs[i](theta[i])

    times, infectedIndices, removedIndices, graph = model.simulate()
    graphFileName = outputDir + "epidemicGraph" + str(j)
    graph.save(graphFileName)
    logging.info("Saved epidemic graph to file " + graphFileName)

    infectIndicesFileName = outputDir + "epidemicInfectIndices"  + str(j) + ".pkl"
    Util.savePickle(infectedIndices, infectIndicesFileName)
    
    detectIndicesFileName = outputDir + "epidemicDetectIndices"  + str(j) + ".pkl"
    Util.savePickle(removedIndices, detectIndicesFileName)

    timesFileName = outputDir + "epidemicTimes" + ".pkl"
    Util.savePickle(times, timesFileName)




